/**
 * Usage:
 *
 * node fixes/get_population_map_object.js INPUT_FILE [EXTRA_ARGUMENTS]
 *
 * Example:
 * node fixes/get_population_map_object.js data/ENTIDADES.csv 2022 ""
 */

const fs = require('fs')
const csv = require('csv-parser')

const [, , input, ...args] = process.argv

const argv = key => {
  // Return true if the key exists and a value is defined
  if (process.argv.includes(`--${key}`)) return true

  const value = process.argv.find(element => element.startsWith(`--${key}=`))

  // Return null if the key does not exist and a value is not defined
  if (!value) return null

  return value.replace(`--${key}=`, '')
}

const filterFn = argv('filter') || false
const additionalFields = filterFn ? args.filter(x => !x.includes('filter')) : args

function groupBy (data, key) {
  return data.reduce((acc, x) => {
    const cat = key instanceof Function ? key(x) : x[key];
    (acc[cat] = acc[cat] || []).push(x)
    return acc
  }, {})
};

const results = []
fs.createReadStream(input)
  .pipe(csv({
    separator: ';',
    skipLines: 1,
    headers: ['ref:ine', , , , , 'population'],
    // parsear los tipos de datos del archivo
    mapValues: ({ header, value }) => ['population'].includes(header) ? Number(value.replace(/,/, '.')) : value
  }))
  .on('data', (data) => {
    if (filterFn) {
      data['ref:ine'].startsWith(filterFn) && results.push(data)
    } else {
      results.push(data)
    }
  })
  .on('end', () => {
    const entities = results.map((item) => [item['ref:ine'], [item.population, ...additionalFields]])

    const BY_MUNICIPALITY = ([a]) => a.slice(0, 5)
    const ENTIDAD_SINGULAR = ([key]) => key.endsWith('00')
    const filtered = entities.filter(ENTIDAD_SINGULAR)

    Object.entries(groupBy(filtered, BY_MUNICIPALITY)).forEach(([key, value]) => {
      fs.writeFile(`${key}.json`, JSON.stringify(Object.fromEntries(value), null, 2), () => { })
    })
  })
