import layers from 'josm/layers'
import { buildChangeCommand } from 'josm/command'
import * as console from 'josm/scriptingconsole'

function tag_from_map () {
  const layer = layers.activeLayer
  if (layer == null) return

  const op = Java.type('javax.swing.JOptionPane')
  const selection = Array.from(layer.data.getSelected())
  if (!selection.length) {
    op.showMessageDialog(null, 'The selection does not contain any item. Please, select something before.', 'No items selected', op.ERROR_MESSAGE)
    return
  }

  const tags = [...selection.reduce((acc, elem) => { elem.getKeys().keySet().forEach(x => acc.add(x)); return acc }, new Set()), '@id'].sort()

  const from = op.showInputDialog(null, 'Which tag you want to pivot?', '', op.QUESTION_MESSAGE, null, tags, tags[0])
  if (from === null) return

  const to = op.showInputDialog('Add the new tag (or tags, separated by comma)').split(',').map(x => x.trim())
  if (to === null) return
  if (to == '') {
    op.showMessageDialog(null, 'You need to type at least one tag, in order to see the results.', 'No output tag', op.ERROR_MESSAGE)
    return
  }

  const textArea = new javax.swing.JTextArea(10, 20)
  op.showConfirmDialog(null, new javax.swing.JScrollPane(textArea), 'Add the javascript map object', op.OK_CANCEL_OPTION)
  const map = textArea.getText()
  if (map === null) return
  if (map == '') {
    op.showMessageDialog(null, 'You need to paste a valid JSON object, in order to apply the changes', 'No map object', op.ERROR_MESSAGE)
    return
  }

  const results = []

  try {
    const parsedMap = JSON.parse(map)

    for (let index = 0; index < selection.length; index++) {
      const element = selection[index]
      const keyMap = element.getKeys()

      // include the element_id, as well as the resto of the keys
      keyMap.put('@id', element.getUniqueId().toString())
      const pivot = keyMap[from]

      if (pivot) {
        const mapped = parsedMap[pivot]

        if (mapped) {
          const values = Array.isArray(mapped) ? mapped : [mapped]
          results.push([pivot, ...to.flatMap((tag, ix) => [keyMap[tag], values[ix]])])
          buildChangeCommand(element, { tags: to.reduce((acc, tag, ix) => ({ ...acc, [tag]: values[ix].toString() }), {}) }).applyTo(layer)
        }
      }
    }
  } catch (e) {
    console.println(e)
  }

  // Results panel
  const panel = new javax.swing.JPanel()
  panel.add(new javax.swing.JScrollPane(new javax.swing.JTable(results, [from, ...to.flatMap(t => [`${t}: from`, `${t}: to`])])))
  op.showMessageDialog(null, panel, 'Results', op.INFORMATION_MESSAGE)
}

tag_from_map()
