/**
 * Usage:
 * 
 * node fixes/update_osm_file_from_map.js "data/file.osm" "data/map.json" ref:ine population population:date source:population
 */
const fs = require('fs')
const xml2js = require('xml2js')
const prompts = require('prompts');
const FuzzySet = require('fuzzyset');

const [, , file, map, pivot, ...updates] = process.argv;

const isFuzzy = false

function getId(element) {
  return element["$"].id
}

function hasTag(tag, element) {
  return element?.tag?.some(({ $ }) => $.k === tag)
}

function getTag(tag, element) {
  return element?.tag?.find(({ $ }) => $.k === tag)
}

// function getTagKey(tag, element) {
//   return getTag(tag, element)?.["$"]?.k
// }

function getTagValue(tag, element) {
  return getTag(tag, element)?.["$"]?.v
}

function setTag(tag, value, element) {
  const ix = element?.tag?.findIndex(x => x["$"]?.k === tag)

  const { tag: tags } = element
  if (ix > -1) {
    // if value is informed, replace tag. when the tag is an empty string, remove tag
    typeof value === "string" && (value.length ? tags.splice(ix, 1, { "$": { k: tag, v: value }}) : tags.splice(ix, 1)) && (element["$"].action = "modify")
  } else {
    // add tag only if value is informed
    !!value && tags.push({ "$": { k: tag, v: value }}) && (element["$"].action = "modify")
  }

  element.tag = tags

  return element
}

function setTags(tags, element) {
  return tags.reduce((acc, [k,v]) => setTag(k,v,acc), element)
}

fs.readFile(file, 'utf-8', (err, data) => {
  if (err) throw err

  xml2js.parseString(data, (err, result) => {
    if (err) throw err

    fs.readFile(map, 'utf-8', async (err, comparator) => {
      if (err) throw err

      const MAP = JSON.parse(comparator);
      const { osm: { node = [], way = [], relation = [] } } = result;

      const updater = async (elem, set) => {
        const ix = set.findIndex(x => getId(x) === getId(elem))
        if (ix > -1) {
          let value = MAP[getTagValue(pivot, elem)]

          // In case there's no matches, ask to the user
          if (isFuzzy && !value) {
            const term = FuzzySet() // order the matches by distance
            term.add(getTagValue(pivot, elem))
            const options = Object.keys(MAP).map(x => {
              const distance = term.get(x, [[0,0]])?.flat()[0]
              return { title: `${x} [${distance?.toLocaleString(undefined, { style: "percent" })}]`, value: x, distance }
            }).sort((a,b) => a.distance < b.distance ? 1 : -1)
            const { value: chosen } = await prompts({
              type: 'select',
              name: 'value',
              message: `Not found: ${getTagValue(pivot, elem)}`,
              choices: [...options, { title: "Ignore and continue...", value: undefined }],
              initial: 0
            })
            value = MAP[chosen]
          }

          set.splice(ix, 1, setTags(updates.map((t, i) => [t, (Array.isArray(value) ? value : [value])?.[i]]), elem))
        }
      } 

      // Chain promises to ask user one by one
      await node.filter(x => hasTag(pivot, x)).reduce((prev, elem) => prev.then(() => updater(elem, node)), Promise.resolve(null))
      await way.filter(x => hasTag(pivot, x)).reduce((prev, elem) => prev.then(() => updater(elem, way)), Promise.resolve(null))
      await relation.filter(x => hasTag(pivot, x)).reduce((prev, elem) => prev.then(() => updater(elem, relation)), Promise.resolve(null))

      result.osm.node = node
      result.osm.way = way
      result.osm.relation = relation

      const builder = new xml2js.Builder()
      const xml = builder.buildObject(result)

      fs.writeFile(file, xml, err => {
        if (err) throw err
      })
    })
  })
})