/**
 * Uso:
 *
 * node fixes/fuzzy_search.js <fichero_osm> <diccionario>
 */

// En caso de que una simple transformación facilitase el matcheo, podemos editar la siguiente función sobre los valores del diccionario
// const transform = entry => entry.replace(/\(.*\)/g, '').trim()
const transform = false

const fs = require('fs')
const path = require('path')
const prompts = require('prompts')
const FuzzySet = require('fuzzyset')

const [, , rawFile, rawDictionary] = process.argv

// Helpers
const store = new Map()
const doTransform = (cb) => {
  if (!(transform instanceof Function)) return cb

  return cb.map(entry => {
    const modified = transform(entry)
    store.set(modified, entry)
    return modified
  })
}
const undoTransform = (cb) => {
  if (!(transform instanceof Function)) return cb

  return cb.map(([key, value]) => [key, store.get(value)])
}
const partition = (arr = [], condition = () => {}) => arr.reduce(([pass, fail], elem) => (condition(elem) ? [[...pass, elem], fail] : [pass, [...fail, elem]]), [[], []])

// function clean(str = "") {
//   return str.toLowerCase().normalize("NFD").replace(/[\u0300-\u036f]/g, "").replace(/\b(de|del|el|la|los|las|y|e|o|u)\b/g, "").split(" ").filter(Boolean)
// }

// function best(str = "", opts = [], THRESHOLD = 0.75) {
//   if (!str) return

//   const _str = clean(str);
//   const _opts = opts.map(clean);
//   const w = _opts.map((x,i) => _str.filter(y => x.filter(h => h.length > 3).includes(y)).length);

//   if (w.every(x => x < (_str.length * THRESHOLD))) return
//   return opts[w.indexOf(Math.max(...w))]
// }

fs.readFile(rawFile, 'utf-8', (err, file) => {
  if (err) throw err

  fs.readFile(rawDictionary, 'utf-8', async (err, dictionary) => {
    if (err) throw err

    const rows = file.split('\n').filter(Boolean)
    const questions = rows.map(target => {
      const term = FuzzySet({ useLevenshtein: false })
      doTransform(dictionary.split('\n')).map(entry => term.add(entry))

      const matches = term.get(target, null, 0.3)

      if (!matches) return null

      const options = matches.flatMap(([distance, term]) => ({ title: `${term} [${distance.toLocaleString(undefined, { style: 'percent' })}]`, value: term, distance }))

      return {
        type: 'select',
        name: target,
        message: `Similar matches for: ${target}`,
        choices: [...options, { title: 'Ignore and continue...', value: null }],
        // Se puede resolver automaticamente si cumple que:
        // solo existe una respuesta cuya distancia es 1 (el término es igual al diccionario)
        autosolve: options.filter(x => x.distance === 1).length === 1
      }
    }).filter(Boolean)

    const [auto, notAuto] = partition(questions, x => x.autosolve)
    const autoResponses = auto.map(x => [x.name, x.choices[0].value])
    const notAutoResponses = await prompts(notAuto.map((x, ix) => ({ ...x, message: `[${ix + 1}/${notAuto.length}] ${x.message}` })))

    const responses = [...autoResponses, ...Object.entries(notAutoResponses)].filter(([key, value]) => value)
    const result = Object.fromEntries(undoTransform(responses))

    // voltea el resultado de un objeto: { key: value } -> { value: key }
    //
    // function flip(obj) {
    //   const ret = {};
    //   Object.keys(obj).forEach(key => {
    //     ret[obj[key]] = key;
    //   });
    //   return ret;
    // }

    console.log(JSON.stringify(result))

    // imprime en fichero
    fs.writeFile(`${path.parse(rawFile).name}.json`, JSON.stringify(result), () => {})

    return result
  })
})
